;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Loading this file installs CG/JS.
;; Please see readme.txt for more information
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(in-package :user)

(eval-when (compile eval load)
  (require :shell)
  (use-package :excl.shell))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; validate files

(let ((platform-file
       (directory
	(merge-pathnames
	 "PLATFORM-*"
	 (path-pathname *load-pathname*)))))
  (or platform-file
      (error "Could not find the PLATFORM file."))
  
  (when (> (length platform-file) 1)
    (error "There is more than one PLATFORM file: ~a." platform-file))
  (setq platform-file (car platform-file))
  
  (or (=~ "^PLATFORM-(.*)" (file-namestring platform-file))
      (error "Cannot parse the PLATFORM file."))
  
  (let ((expected-platform $1)
	(current-platform
	 (format nil "~a~a" (symbol-name sys::.os.) #+smp "smp" #-smp "")))
    (or (string-equal expected-platform current-platform)
	(error
	 "The current platform (~a) does not match the downloaded version (~a)."
	 current-platform expected-platform)))
  ;; We are on the correct platform, fall through...
  )

;; HTML files needed for the operation of CG/JS
;; (this variable's value is generated by build_cgjs.sh)
(defvar *html-files* '("cgjs.html" "ssl-not-found.html" "browser-not-supported.html" ))

(defvar *extra-files*)
(setq *extra-files* nil)

(dolist (file *html-files*)
  (setq file (merge-pathnames file *load-pathname*))
  (or (probe-file file)
      (error "~a does not exist." file))
  (push file *extra-files*))
;; All *html-files* exist...

;; These two are directories but the pathname is specified as
;; a file, so merging works below.
(defvar *cg2js-directory* (merge-pathnames "code.cg2js" *load-pathname*))
(defvar *idejs-directory* (merge-pathnames "code.idejs" *load-pathname*))

(dolist (d (list *cg2js-directory* *idejs-directory*))
  (or (probe-file d)
      (error "~a does not exist." d)))

;; MD5 checksums of the incoming files
;; (this variable's value is generated by build_cgjs.sh)
(defvar *checksums* 
'(
(#x6ce3bbba897a069af3d6cb40385de645 . "./code.cg2js/cg-acache.fasl")
(#x6899f5b5ccbfe193cae01ed715de29fd . "./code.cg2js/cg-base.fasl")
(#x145ba4859cc2555d24d305e6e8e12c26 . "./code.cg2js/cg-bitmap-pane-clipboard.fasl")
(#x60179ed9c8f963dfe043ef27186f9f3a . "./code.cg2js/cg-bitmap-pane.fasl")
(#x4112108b999518a91f3cf50478b4617d . "./code.cg2js/cg-bitmap-stream.fasl")
(#x01c3cff8ba0180cce7d7ed1ad22ac114 . "./code.cg2js/cg-button.fasl")
(#xe586fdb5cecaf14a184e16601fe7a635 . "./code.cg2js/cg-calendar.fasl")
(#x423a3d28079151f27e0df1880371b499 . "./code.cg2js/cg-caret.fasl")
(#xca0226c2ce3a6cbb00ccde6cf3d3b6e4 . "./code.cg2js/cg-chart-or-plot.fasl")
(#xa77ef4430289646d17d2506013816cdd . "./code.cg2js/cg-chart-widget.fasl")
(#x2acd84d2c527272b3e82f88c52032d4f . "./code.cg2js/cg-check-box.fasl")
(#x689d907dae1735b9ae5efded544f59ad . "./code.cg2js/cg-choice-list.fasl")
(#x5ca5843550637dcf1cbe8bb7ea75c7fd . "./code.cg2js/cg-choose-printer.fasl")
(#x24b843e021e78e06bac7283e43fb3931 . "./code.cg2js/cg-class-grid.fasl")
(#x0c41281da3897175761fe1dd35fbe8f0 . "./code.cg2js/cg-class-slot-grid.fasl")
(#x87e5140c7d6ef334fe155a58ccbc202c . "./code.cg2js/cg-class-support.fasl")
(#x130d2b0acdba5fbb72413eba0f3dabee . "./code.cg2js/cg-clipboard-pixmap.fasl")
(#xc092c823654f15a52e18cf0a4ae314f8 . "./code.cg2js/cg-clipboard-stack.fasl")
(#x63577e79c59a55e2ec2091c2cf97ad94 . "./code.cg2js/cg-clipboard.fasl")
(#x02d80aa4c900795e6530dbeb69eb2cb5 . "./code.cg2js/cg-color-dialog.fasl")
(#x3e19517b5713bcf47c89a7bd2a7c6b8e . "./code.cg2js/cg-combo-box.fasl")
(#xb5be2bb0d3154e9a2b664330524433b6 . "./code.cg2js/cg-common-control.fasl")
(#xd1349a718c37063a33f6a14dc8f5e88e . "./code.cg2js/cg-comtab.fasl")
(#x9ed73af7436024c802b6f0cdf54dbb63 . "./code.cg2js/cg-cursor-pixmap.fasl")
(#xa49c286f3abda0a9589d7412d63c6edf . "./code.cg2js/cg-curve.fasl")
(#xf53041fa904611d94d498574c60ab7c6 . "./code.cg2js/cg-dde-utils.fasl")
(#x506e3a3ad149d19812a7c1ced059d108 . "./code.cg2js/cg-dev.fasl")
(#xd97ad9c8e79e54a143da806bb01c359e . "./code.cg2js/cg-dialog-item.fasl")
(#x07f9f29a5c672aa4bb97b67dc4dd3610 . "./code.cg2js/cg-directory-dialog-os.fasl")
(#xd02df26ddd771cd83ba6c1e35bc65d9d . "./code.cg2js/cg-directory-dialog.fasl")
(#xee03aa1e95b1f9fdf377e1a0fac9a3cb . "./code.cg2js/cg-drag-and-drop-image.fasl")
(#xe821ae7a3cfb7c60dff26cd117020f1e . "./code.cg2js/cg-drag-and-drop.fasl")
(#xdd90e1171ae97971a46cf873896daff2 . "./code.cg2js/cg-drawable-clipboard.fasl")
(#x32000b4ea133eb241dafcbf39c244cff . "./code.cg2js/cg-drawable.fasl")
(#x5f3ab3f7331c8e54ef821656d829267a . "./code.cg2js/cg-dropping-outline.fasl")
(#x18b98b618aa4746e04b4e59cfc99c185 . "./code.cg2js/cg-edit-in-place.fasl")
(#x44e638f67cbd6a7d0d542592b4c411af . "./code.cg2js/cg-editable-text.fasl")
(#x9c04caa8f86196b6e7d5cfa43735b756 . "./code.cg2js/cg-file-dialog.fasl")
(#x0a59a364aff4509514aba018b7d94e53 . "./code.cg2js/cg-fill-texture.fasl")
(#x9856d073aa4f72e3431d55796f417e28 . "./code.cg2js/cg-find-string-dialog.fasl")
(#x578aa04d73ecd221b1f6283f2760f007 . "./code.cg2js/cg-font-dialog.fasl")
(#x234aeb56a1e315155c751c4b26c21a4f . "./code.cg2js/cg-gesture-emulation.fasl")
(#x46e094f7bfcc41eca03cd12812370ef1 . "./code.cg2js/cg-get-pixmap.fasl")
(#xaad4ed5bfb9fa3d64c165d6f30f9211b . "./code.cg2js/cg-get-position.fasl")
(#x471cf670f7139a56c36997c3cfdf54ef . "./code.cg2js/cg-graphics-context.fasl")
(#x4b9ffaaf4c0fdf7cbd44022a3e78878d . "./code.cg2js/cg-grid-widget-drag-and-drop.fasl")
(#x3ed055807d591ba5f5da4cdf0c6d7abe . "./code.cg2js/cg-grid-widget.fasl")
(#x702c0a88eadf4daff0afe12277019af2 . "./code.cg2js/cg-group-box.fasl")
(#xf0bcb09ecfc2f3fcdcff1504c9f8cc84 . "./code.cg2js/cg-header-control.fasl")
(#x1fa14083f4cab9cea99236c01013e366 . "./code.cg2js/cg-hotspot.fasl")
(#x556ce5f9db0c3c047cd5cca3cf86860c . "./code.cg2js/cg-html-dialog.fasl")
(#x6459ae0ed119ee5017818eec682f4951 . "./code.cg2js/cg-html-widget.fasl")
(#xd2bc8a666c3239eb21e7ecb119dc9bf8 . "./code.cg2js/cg-icon-pixmap.fasl")
(#x2e30e4fd35d726250dfad8645717e517 . "./code.cg2js/cg-icon.fasl")
(#x58d5ffa95b38f509fd5dd29a87f76430 . "./code.cg2js/cg-ie.fasl")
(#x344950bc87ae076d7e1c86b6a0941bd6 . "./code.cg2js/cg-intersect-posbox.fasl")
(#xca407a878a2c1e52906dba075fd7f618 . "./code.cg2js/cg-intersect.fasl")
(#x22e04f2e173d5194aec5b5fb635b6fe0 . "./code.cg2js/cg-item-list.fasl")
(#x9973d39f9a08dba181bb99cce597f401 . "./code.cg2js/cg-keyboard-shortcuts.fasl")
(#x2f9b5df57bc4b25743554a6786e32b3f . "./code.cg2js/cg-lamp.fasl")
(#xe5fa4fdb0f7352d2f25b67172406a8e1 . "./code.cg2js/cg-layout.fasl")
(#x8b1fa4e5a0e2bf5d03e899394e98263e . "./code.cg2js/cg-lettered-menu.fasl")
(#xc0c516b4101d9ac8de1bcc41e70d1956 . "./code.cg2js/cg-lisp-edit-pane.fasl")
(#x26b1362d4954d0b1bd0593ae226e53b6 . "./code.cg2js/cg-lisp-text.fasl")
(#x4891a65b6fc917772f1e36abb037c7ca . "./code.cg2js/cg-lisp-widget.fasl")
(#xb822ee56813d564aa0222f7b77adc50e . "./code.cg2js/cg-list-view.fasl")
(#xb9ad19b3de1370b203fc486e98898286 . "./code.cg2js/cg-mci.fasl")
(#x596986b543aa0ffc5a28d7dc413d3435 . "./code.cg2js/cg-menu-tooltip.fasl")
(#x8e39bda05ed68beb238cc97c552642b8 . "./code.cg2js/cg-menu.fasl")
(#x9cac93f54f561cfa3408e97edffff24f . "./code.cg2js/cg-message-dialog.fasl")
(#x8d6c00ceeb744517e67495fab3690a42 . "./code.cg2js/cg-multi-line-editable-text.fasl")
(#x055de71229be39db8381293e878a9ff9 . "./code.cg2js/cg-multi-line-lisp-text.fasl")
(#xabcbe564476e6cda537ca86d758ecbab . "./code.cg2js/cg-multi-picture-button-drag-and-drop.fasl")
(#x630505415273b227e63bfebed34005cf . "./code.cg2js/cg-multi-picture-button-tooltip.fasl")
(#x85ac5f2597b6cb1e683662dcad90b57f . "./code.cg2js/cg-multi-picture-button.fasl")
(#xfc018ac4e5fa0ddc27df18133809f824 . "./code.cg2js/cg-nodes.fasl")
(#xea4e450cb21d744cd4702170785d546c . "./code.cg2js/cg-object-editor-layout.fasl")
(#xff554701cd7216bc0e2ad8d8efbf1b60 . "./code.cg2js/cg-object-editor.fasl")
(#x9e83e8a0754f116390c779707d1d3cde . "./code.cg2js/cg-ocx.fasl")
(#xd61afd1e404985a693a59100593f1713 . "./code.cg2js/cg-os-widget.fasl")
(#x284148848a2c1405510b32d17c744ac8 . "./code.cg2js/cg-os-window.fasl")
(#xdee29ecbe04b6e678a37196fdab1dae4 . "./code.cg2js/cg-outline-drag-and-drop.fasl")
(#x995f08f70026e065ab6032e591aa7583 . "./code.cg2js/cg-outline-edit-in-place.fasl")
(#x0da30629a6fd2ee1131add7a284d3cb5 . "./code.cg2js/cg-outline.fasl")
(#x7220cb4184611d0e4d4b96aefc8450f3 . "./code.cg2js/cg-palette.fasl")
(#x67f9a4ef2e00e2dc6c9f483e5f600be1 . "./code.cg2js/cg-paren-matching.fasl")
(#xeb433e3e64579e10db7f9efb3506f07b . "./code.cg2js/cg-picture-widget-palette.fasl")
(#x3c65ae0d3958f0a725eebf6409618adf . "./code.cg2js/cg-picture-widget.fasl")
(#xe38b1cfcd47c7820c29598f47b43cdfa . "./code.cg2js/cg-pixmap-file-io.fasl")
(#x83db40e5d076f8726ae1897f8da30c2d . "./code.cg2js/cg-pixmap-printing.fasl")
(#x32a70ff8066582c330097e64a3a12af8 . "./code.cg2js/cg-pixmap-rotate.fasl")
(#x2e18bda7284f6426ed4fa5e8dc513f10 . "./code.cg2js/cg-pixmap-widget.fasl")
(#xeff5d9882d6ffc550eb2cddc49edd6a5 . "./code.cg2js/cg-pixmap.fasl")
(#xca55e3e04bf6dd4a0efda78aef4d99b5 . "./code.cg2js/cg-player.fasl")
(#xea4c632e44065964fe54028c12e78bd0 . "./code.cg2js/cg-plot-widget.fasl")
(#x07d5cd75f6b3da55e78ea18ef73d31e2 . "./code.cg2js/cg-printing.fasl")
(#xbb58372ce4db3002206fca2900da33b9 . "./code.cg2js/cg-progress-indicator.fasl")
(#xc6a656018b74f6b38edb5a435bf6e35d . "./code.cg2js/cg-project-window.fasl")
(#x72928ed4ee5ed85b463abde4b148c7ac . "./code.cg2js/cg-property.fasl")
(#x44a3c325e952052f71f52906211b8bbd . "./code.cg2js/cg-radio-button.fasl")
(#x6490633115be15002152858bcc0892bd . "./code.cg2js/cg-rich-edit-pane-clipboard.fasl")
(#xf60808654f94a1ac2c1c77b069fcee28 . "./code.cg2js/cg-rich-edit-pane-printing.fasl")
(#xfb0bc37059a158b027dc80b5dec76dda . "./code.cg2js/cg-rich-edit-pane.fasl")
(#xb19ef879bc6415dcfa7ffa993e991320 . "./code.cg2js/cg-rich-edit.fasl")
(#xfe0be1abafa5e825d9d7650d01850a8a . "./code.cg2js/cg-sample-file-menu.fasl")
(#x869eb05086ab5ec3578cf9c608019ac6 . "./code.cg2js/cg-scaling-stream.fasl")
(#x6f543697d0458d66f84bbf5ff6e55d66 . "./code.cg2js/cg-scroll-bar-mixin.fasl")
(#x4af5d562938300dd780a73466a8e3e32 . "./code.cg2js/cg-scroll-bar.fasl")
(#xcd07da4e9e6fb13d3316ffbeb7e5ee13 . "./code.cg2js/cg-scrolling-static-text.fasl")
(#x4ade898599e803250c3a3f2974cd882a . "./code.cg2js/cg-selected-object.fasl")
(#xa2d01bc828586623d0929d8047c066eb . "./code.cg2js/cg-shortcut-menu.fasl")
(#xffe5bc22c15a8a96c0d68736e89f8cec . "./code.cg2js/cg-split-bar.fasl")
(#x2daf3c9bdae7e1c1e5f219cf1fa6d6c0 . "./code.cg2js/cg-static-text.fasl")
(#xabbf6489f437b1bc137c801c38602542 . "./code.cg2js/cg-status-bar.fasl")
(#x723974e36381998fd23f915fcfbee326 . "./code.cg2js/cg-string-dialog.fasl")
(#xaafa0eafc3e4de9203a29ef99f8f72e6 . "./code.cg2js/cg-tab-control.fasl")
(#x6eeb8092e3295f60237620577e53c7c7 . "./code.cg2js/cg-template-string.fasl")
(#x53a67b126855b854c28ed0c9a883360e . "./code.cg2js/cg-text-edit-pane-file-io.fasl")
(#xc108d5f308cf79c11c693f31b44a70ed . "./code.cg2js/cg-text-edit-pane-mark.fasl")
(#x86684f1d41f4816edd0245be8890942c . "./code.cg2js/cg-text-edit-pane.fasl")
(#x7ba6860ccd66b5512d6e2d14443aaf62 . "./code.cg2js/cg-text-or-combo.fasl")
(#x8b077ddf0c21392c98446cf26fc63ae1 . "./code.cg2js/cg-text-widget.fasl")
(#x9ab304d8e940e0772f090bb00dee4c48 . "./code.cg2js/cg-timer.fasl")
(#xc10d0698da1c9022dcfcfbe0371fb9b3 . "./code.cg2js/cg-toggling-widget.fasl")
(#xf2d355f496250f78b34e01616444e8d3 . "./code.cg2js/cg-toolbar.fasl")
(#x50424cc5881abb79fae702f28de0d043 . "./code.cg2js/cg-tooltip.fasl")
(#x4cd894f7ca3b134291b75bda6afe4a5a . "./code.cg2js/cg-touch.fasl")
(#x1fe9a308ab4cb837f58a350c34918989 . "./code.cg2js/cg-trackbar.fasl")
(#x387a3d38ae06cffedb1de1e99ed1f72e . "./code.cg2js/cg-tray.fasl")
(#xb1f11cdf3ebd48451833d06ec9439f39 . "./code.cg2js/cg-up-down-control.fasl")
(#x95e5d567fb40de8d6ea046a803405dcb . "./code.cg2js/cg-utility-dialog.fasl")
(#x7136cc74a7920e9c346391d03125f0ae . "./code.cg2js/cg-web-browser-dde.fasl")
(#xd3355c1420301ac2917cdad7624b5218 . "./code.cg2js/cg-web-browser.fasl")
(#x603c38dba5fdafafcde7c176db9a8d28 . "./code.cg2js/cg-wrap-string.fasl")
(#x7c6a57bf8261b4adf6ebc1295015afac . "./code.cg2js/cg-yes-no-list.fasl")
(#x8060d497001042d4ec892c3339a8d827 . "./code.cg2js/cg.fasl")
(#x7a2e78077aea10ef7b46940e352513f8 . "./code.cg2js/cg-yes-no-string.fasl")
(#x6c6715cbeb549bb8d8fc39c1457e5c5c . "./code.idejs/ide-acache.fasl")
(#x36a6608ea01693aa38053a9537f45552 . "./code.idejs/ide-apropos.fasl")
(#xb885784a6a0fcbfc546295c007f7c486 . "./code.idejs/ide-base.fasl")
(#xead976c8c2e38328141c067042e48881 . "./code.idejs/ide-class-browser.fasl")
(#xb43e09d8a4cb9630c2dec1f142bb36f5 . "./code.idejs/ide-class-grapher.fasl")
(#x9a71edc4f4aef596d85f9e14e12f3038 . "./code.idejs/ide-clipboard.fasl")
(#x6b2cf4a175f0f00a80ad7dbb51afcbc9 . "./code.idejs/ide-dev.fasl")
(#x7e797d8fab8e25e45f762f34f7ec3ba8 . "./code.idejs/ide-editor.fasl")
(#xb62023494858036dff142433ef4f928b . "./code.idejs/ide-find-definitions.fasl")
(#xf6b5bd59a1b81d2062a78c3602d629d2 . "./code.idejs/ide-find-in-files.fasl")
(#xebdd619e55d6f5d33b48038e2647cd1e . "./code.idejs/ide-form.fasl")
(#xf0c40f70c745f6e387378e6b2911c58a . "./code.idejs/ide-grapher.fasl")
(#x7bf0aa897c231a37b7097d7ee7e3c043 . "./code.idejs/ide-inspector.fasl")
(#x87f25f924534f4be3f4e8b73f6a7e33d . "./code.idejs/ide-intro.fasl")
(#x7c8a0b010e4d5250e35c2925be871b70 . "./code.idejs/ide-keyshort.fasl")
(#x9a27ef62b9295888a723cbb0956aa35e . "./code.idejs/ide-navigator.fasl")
(#xbe9da0422a7dfa9dc00b6653cc158e4e . "./code.idejs/ide-package-list.fasl")
(#x7305f732546c81d9870db2c25825cccb . "./code.idejs/ide-patch-dialog.fasl")
(#x7fbde050789adf0ed852edaab7244480 . "./code.idejs/ide-process-browser.fasl")
(#x3d8efdafbe47b3055750ea1602fc04ba . "./code.idejs/ide-project.fasl")
(#x4b174afb59352706492fcbd0747ef70b . "./code.idejs/ide-trace.fasl")
(#x03becac42a66592d832ffd79e60631bd . "./code.idejs/ide-window-list.fasl")
(#xa41eb7083870f0ed6d2ba3aec1f073d1 . "./code.idejs/ide.fasl")
(#x3d70f7088b7ddd139f75c06fecd8ac58 . "./cgjs.html")
(#x55fea08d92e113f5a8381c271febbc9e . "./ssl-not-found.html")
(#x4e1b21622d7d9cc4e6fc09c7875f8430 . "./browser-not-supported.html"))
 )

(let ((files-ok
       (dolist (checksum *checksums* t)
	 (when (/= (car checksum)
		   (md5-file
		    (merge-pathnames (cdr checksum) *load-pathname*)))
	   (return nil)))))
  (cond
   ((not files-ok)
    (error "The extracted files failed the checksum test.  ~
Please download and/or extract again."))
   (t
    (format t "Checksums on files are OK...~%"))))

;; We have validated all the files are there...

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; check for needed patches

(eval-when (compile eval load) (require :build))

(defvar *required-patches* '(#+macosx ("build" . 3)))

(dolist (required-patch *required-patches*)
  (let ((existing-patch (assoc (car required-patch)
			    (cdr (assoc :system sys::*patches*))
			    :test #'string=)))
    (when (null existing-patch)
      (error "Could not find patch info for the ~s module."
	     (car required-patch)))
    (when (not (>= (second existing-patch) (cdr required-patch)))
      ;; The build patch number is NOT >= to the one we require
      (error "~
Expected version ~d of ~s but have version ~d.  Please update ~
your lisp with update.~a and then do the CG/JS installation again."
	     (cdr required-patch) (car required-patch)
	     (second existing-patch)
	     #+mswindows "exe"
	     #-mswindows "sh"))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; copy files

(defun copy (source dest)
  (cond
   ((file-directory-p source)
    (format t "Copy directory ~a~%" (enough-namestring source))
    (copy-directory source dest :quiet t))
   (t
    (format t "Copy file ~a~%" (enough-namestring source))
    (sys:copy-file source dest :verbose nil))))

(defvar *sys* (translate-logical-pathname (pathname "sys:")))

(cond
 ((excl::pathname-equalp (truename *sys*)
			 (path-pathname *load-truename*))
  (format t "Files are already in place, no copying requiring.~%"))
 (t ;; copy files, replacing them with new ones if they already exist
  (let (dest)
    (dolist (from *extra-files*)
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      (when (probe-file dest) (delete-file dest))
      (copy from dest))
    
    (dolist (from (list *cg2js-directory* *idejs-directory*))
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      ;; now make it a directory pathname
      (setq dest (pathname-as-directory dest))

      (when (probe-file dest) (delete-directory-and-files dest))
      (copy (pathname-as-directory from) dest)))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; update develenv.cl to load CG/JS before the original version of CG

(defvar *setup-cgjs-loading* "sys:setup-cgjs-loading.cl")

;; Because users might use an ANSI lisp to install, have to make sure
;; the resulting file can be loaded into an ANSI and Modern lisp.
(with-open-file (s *setup-cgjs-loading* :direction :output
		 :if-exists :supersede)
  (format s "~
;; This file was created by install-cgjs.cl.
;; Please do not edit it.

")
  (dolist (dir (list "code.cg2js" "code.idejs"))
    (format s "
(let ((path (make-pathname
             :host \"sys\"
             :directory '(:relative \"~a\")
	     :type \"fasl\"
	     )))
  (format t \";; *require-search-list*: adding: ~~s~%\" path)
  (push path sys:*require-search-list*))

"
	    dir)))

(defvar *develenv.cl* (pathname "sys:develenv.cl"))

(when (not (probe-file *develenv.cl*))
  (setf (file-contents *develenv.cl*)
    ";; This file is loaded before images are rebuilt with update.sh/update.exe

"))

(let ((contents (file-contents *develenv.cl*)))
  (cond
   ((match-re
     (format nil "^#\\|\\|# \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl already correctly loads CG/JS.~%"))
   ((match-re
     (format nil "^#\\+ignore \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl: CG/JS disabled; will enable.~%")
    (setf (file-contents *develenv.cl*)
      (replace-re  contents
		  (format nil "^#\\+ignore *\\(load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  (format nil "#||# (load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  :multiple-lines t
		  :count 1)))
   (t
    (format t "develenv.cl: adding CG/JS initialization.~%")
    (setf (file-contents *develenv.cl*)
      (concatenate 'simple-string
	contents
	(format nil "~

;; To disable the use of CG/JS and go back to the originally released
;; (and possibly patched) version of CG/IDE, change the #||# to
;; #+ignore on the following line and the rebuild images by running
;; update.sh or update.exe, depending on your platform:
#||# (load \"sys:~a\")~%~%"
		(file-namestring *setup-cgjs-loading*)))))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; success!

(let ((msg (format nil "~
CG/JS installation successful!

NOTE: to use CG/JS you must rebuild images by running 'update.~a'
      in the Allegro CL directory (~a)."
		   #+mswindows "exe"
		   #-mswindows "sh"
		   *sys*)))
  #-mswindows (format t "~&~%~a~%" msg)
  #+mswindows
  (let ((noninteractive (sys:getenv "ACL_NONINTERACTIVE")))
    (when (or (null noninteractive)
	      (string= "" noninteractive))
      (excl::internal-message-box msg "CG/JS Installation" :type :ok))))
